% function [numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);
% 
% Calculates compensator numerator parameters. Desired CL TF is
%
% GCL = Kpr*num*exp(-s*Tdel)/((1-Kpr*Kc)*denr)


function [numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);

[a1,a2,a3,a4,a5] = vect2param (den);
[b1,b2,b3,b4,b5] = vect2param (num);
[c1,c2,c3,c4,c5] = vect2param (denc);
[e1,e2,e3,e4,e5] = vect2param (denr);


if (order == 2)
	beta = 6*c1*Tdel*e1+6*c2*e1-6*b1*c1*e1-6*b1*Tdel*e1-6*b3+6*c3+6*c1*b1^2-6*b1^3+6*Tdel*b1^2+Tdel^3+6*b1^2*e1-6*b1*e2+6*c1*e2+12*b2*b1-6*c1*b2-6*Tdel*b2+3*c1*Tdel^2+6*Tdel*e2+6*c2*Tdel-3*Tdel^2*b1-6*c2*b1-6*c1*Tdel*b1-6*b2*e1+3*Tdel^2*e1+6*e3;
	alpha = -3*(-2*c1*Tdel*e1-2*c2*e1-2*e3-2*a1*Tdel*b1-2*b1*a2+2*b1*e2+2*a1*Tdel*c1-2*c1*e2+a1*Tdel^2+2*b1*c1*e1+2*Tdel*a2-2*a1*b1*c1+2*b1*Tdel*e1+2*a1*c2-Tdel^2*e1-2*a1*b2+2*a3+2*a2*c1+2*b2*e1-2*Tdel*e2+2*b1^2*a1-2*b1^2*e1);
elseif (order == 1)
   beta = 2*b1^2+Tdel^2+2*e2-2*b2-2*b1*Tdel+2*Tdel*e1-2*b1*e1+2*e1*c1-2*b1*c1+2*c2+2*c1*Tdel;
   alpha = -2*(a1*Tdel-Tdel*e1-a1*b1+b1*e1-e2-e1*c1+a1*c1+a2);
else
   beta = 1;
   alpha = 2;
end;

if (abs(beta) < 1e-30)
   K1 = 2;
   Kc = 2/Kpr;
   beta = 0
else
   K1 = alpha/beta;
   Kc = K1/Kpr;
end;

d1 = 0;
d2 = 0;
d3 = 0;

if (order > 2)
   d1 = (K1*Tdel-K1*b1+a1+K1*c1-e1+e1*K1)/K1;
	d2 = 1/2*(2*Tdel*e1*K1-2*b1*e1*K1+2*c1*K1*e1-2*b1*c1*K1+2*c2*K1+2*Tdel*c1*K1+2*K1*b1^2-2*K1*b2-2*K1*Tdel*b1+K1*Tdel^2+2*e2*K1+2*a1*c1+2*a2+2*Tdel*a1-2*Tdel*e1-2*b1*a1+2*b1*e1-2*e2-2*e1*c1)/K1;
	d3 = 1/6*(6*c2*K1*Tdel-6*c2*K1*b1+3*a1*Tdel^2-6*a1*b2-6*b1*a2+6*Tdel*a2+6*b1^2*a1+6*a3-6*c1*K1*Tdel*b1-6*a1*Tdel*b1-6*b1*a1*c1+6*Tdel*a1*c1-6*K1*b3+K1*Tdel^3-3*K1*Tdel^2*b1-6*K1*Tdel*b2+6*a1*c2+6*a2*c1-6*e3+6*c3*K1-6*c1*K1*b2+3*c1*K1*Tdel^2+6*e3*K1-6*K1*Tdel*b1*e1-6*b1*c1*K1*e1+6*Tdel*c1*K1*e1+6*b2*e1-3*Tdel^2*e1+12*b2*K1*b1-6*b2*e1*K1+3*Tdel^2*e1*K1+6*c2*e1*K1+6*b1^2*e1*K1-6*c2*e1-6*c1*Tdel*e1+6*c1*b1^2*K1+6*c1*b1*e1+6*Tdel*b1^2*K1+6*Tdel*b1*e1+6*K1*Tdel*e2-6*K1*b1*e2+6*K1*c1*e2-6*K1*b1^3-6*b1^2*e1+6*b1*e2-6*Tdel*e2-6*c1*e2)/K1;
   numc = [d3 d2 d1 1];
elseif (order > 1)
   d1 = (K1*Tdel-K1*b1+a1+K1*c1-e1+e1*K1)/K1;
	d2 = 1/2*(2*Tdel*e1*K1-2*b1*e1*K1+2*c1*K1*e1-2*b1*c1*K1+2*c2*K1+2*Tdel*c1*K1+2*K1*b1^2-2*K1*b2-2*K1*Tdel*b1+K1*Tdel^2+2*e2*K1+2*a1*c1+2*a2+2*Tdel*a1-2*Tdel*e1-2*b1*a1+2*b1*e1-2*e2-2*e1*c1)/K1;
   numc = [d2 d1 1];
elseif (order > 0)
   d1 = (K1*Tdel-K1*b1+a1+K1*c1-e1+e1*K1)/K1;
   numc = [d1 1];
else
   numc = [1];
end

