% Test of unstable processes
%
% Process: Kpr*num*exp(-s*Tdel)/den
%
% Direct POSITIVE feedback: Kc*numf/denf
%
% Desired closed-loop function is: Kpr*num*exp(-s*Tdel)/((1-Kpr*Kc)*denr)
%
% where denr is desired denominator
%
% K1 = Kpr*Kc
%
% num=1+b1*s+b2*s^2+b3*s^3+b4*s^4+b5*s^5;		% Process numerator
% den=1+a1*s+a2*s^2+a3*s^3+a4*s^4+a5*s^5;		% Process denominator
% denf=1+c1*s+c2*s^2+c3*s^3;  					% in fact fixed filter of the n-th order (the same as numf)
% denr=1+e1*s+e2*s^2+e3*s^3; 						% desired denominator of the closed-loop
% numf=1+d1*s+d2*s^2+d3*s^3;						% Filter numerator : has to be CALCULATED! 


numcM= [];
KcM = [];
AreasM = [];
PIM= [];


% *****************************************************
%
%  Illustrative example
%
% *****************************************************


% Process
Kpr=1; Tdel=0.5; num=[0 1]; den=[-2 1]; den=conv(den,[0.5 1]); %den=conv(den,[0 1]); 

% Compensator
order=2;Tf=0.1;denc=[Tf 1]; denc=conv(denc,denc); denc=conv(denc,[Tf 1]); %denc=conv(denc,[Tf 1]);


% Desired closed-loop denominator 
denr=[2 1]; denr=conv(denr,[0.5 1]); denr=conv(denr,[0 1]);


[numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);
while (max(size(numc)) < 3)
   numc = conv(numc,[0 1]);
end;
numcM = [numcM;numc];
KcM = [KcM;Kc];
% Only inner compensator

% Simulation parameters
Tscl = 0.01;
Tstopcl = 30;
Tdist = 0;
Kdist = 1;

% Controller parameters
K = 0; Ki = 0; b=1;

sim ('cl_illustrative');

tcl1 = tcl;
ucl1 = ucl;
ycl1 = ycl;
yref1 = yref;


% With controller

% With PI controller

% Simulation parameters
Tscl = 0.01;
Tstopcl = 40;
Tdist = Tstopcl/2;
Kdist = 1;

% Controller parameters
[A0,A1,A2,A3,A4,A5] = numden2areas (Kpr/(1-Kpr*Kc)*num,denr,Tdel);
b=1;
[Ki,K] = area2PI (A0,A1,A2,A3,2);
Ki0=Ki;
K0 = K;
b0 = b;

sim ('cl_unstab');

tclr1 = tcl;
yclr1 = ycl;
uclr1 = ucl;

b=0;
[Ki,K] = area2PI_dist (A0,A1,A2,A3,2);

sim ('cl_unstab');


tcld1 = tcl;
ycld1 = ycl;
ucld1 = ucl;

AreasM = [AreasM;[A0,A1,A2,A3,A4,A5]];
PIM = [PIM;[Ki0,K0,b0,Ki,K,b]];



% ************** Another desired closed-loop TF

% Compensator
order=2;Tf=0.1;denc=[Tf 1]; denc=conv(denc,denc); denc=conv(denc,[Tf 1]); %denc=conv(denc,[Tf 1]);

% Desired closed-loop denominator 
denr=[0.7 1]; denr=conv(denr,[0.7 1]); %denr=conv(denr,[0 1]);


[numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);
while (max(size(numc)) < 3)
   numc = conv(numc,[0 1]);
end;
numcM = [numcM;numc];
KcM = [KcM;Kc];

% Only inner compensator

% Simulation parameters
Tscl = 0.01;
Tstopcl = 30;
Tdist = 0;
Kdist = 1;

% Controller parameters
K = 0; Ki = 0; b=1;

sim ('cl_illustrative');

tcl2 = tcl;
ucl2 = ucl;
ycl2 = ycl;
yref2 = yref;



% With controller

% With controller

% With PI controller

% Simulation parameters
Tscl = 0.01;
Tstopcl = 40;
Tdist = Tstopcl/2;
Kdist = 1;

% Controller parameters
[A0,A1,A2,A3,A4,A5] = numden2areas (Kpr/(1-Kpr*Kc)*num,denr,Tdel);
b=1;
[Ki,K] = area2PI (A0,A1,A2,A3,2);
Ki0=Ki;
K0 = K;
b0 = b;

sim ('cl_unstab');

tclr2 = tcl;
yclr2 = ycl;
uclr2 = ucl;

b=0;
[Ki,K] = area2PI_dist (A0,A1,A2,A3,2);

sim ('cl_unstab');


tcld2 = tcl;
ycld2 = ycl;
ucld2 = ucl;

AreasM = [AreasM;[A0,A1,A2,A3,A4,A5]];
PIM = [PIM;[Ki0,K0,b0,Ki,K,b]];

% Plot results

figure(1);
subplot(221);
plot(tcl1,ycl1,tcl1,yref1,'--');
grid
axis([0 30 -1.5 0])
legend('actual response','desired response')
title('Process G_P output with compensator 1')
xlabel('time [s]')

subplot(222);
plot(tcl2,ycl2,tcl2,yref2,'--');
grid
axis([0 30 -1.5 0])
legend('actual response','desired response')
title('Process G_P output with compensator 2')
xlabel('time [s]')

subplot(223)
plot(tcl1, ucl1)
grid
axis([0 30 -2.5 0])
title('Process G_P input with compensator 1')
xlabel('time [s]')

subplot(224)
plot(tcl2, ucl2)
grid
axis([0 30 -2.5 0])
title('Process G_P input with compensator 2')
xlabel('time [s]')

print -depsc Gp_il_ol.eps



figure(2);
subplot(221);
plot(tclr1,yclr1,tcld1,ycld1,'--');
grid
title('Process G_P output with compensator 1')
xlabel('time [s]')
legend('MOMI','DRMO')

subplot(222);
plot(tclr2,yclr2,tcld2,ycld2,'--');
grid
title('Process G_P output with compensator 2')
xlabel('time [s]')
legend('MOMI','DRMO')

subplot(223)
plot(tclr1, uclr1, tcld1, ucld1,'--')
grid
title('Process G_P input with compensator 1')
xlabel('time [s]')
legend('MOMI','DRMO')

subplot(224)
plot(tclr2, uclr2, tcld2, ucld2,'--')
grid
title('Process G_P input with compensator 2')
xlabel('time [s]')
legend('MOMI','DRMO')

print -depsc Gp_il_cl.eps

% Display results

display('1. numerator, 2. Kc, 3. Areas (A0..A5), 4. PI (Ki, K, b for MIMO & DRMO)')
numcM
KcM
AreasM
PIM