% Test of unstable processes
%
% Process: Kpr*num*exp(-s*Tdel)/den
%
% Direct POSITIVE feedback: Kc*numf/denf
%
% Desired closed-loop function is: Kpr*num*exp(-s*Tdel)/((1-Kpr*Kc)*denr)
%
% where denr is desired denominator
%
% K1 = Kpr*Kc
%
% num=1+b1*s+b2*s^2+b3*s^3+b4*s^4+b5*s^5;		% Process numerator
% den=1+a1*s+a2*s^2+a3*s^3+a4*s^4+a5*s^5;		% Process denominator
% denf=1+c1*s+c2*s^2+c3*s^3;  					% in fact fixed filter of the n-th order (the same as numf)
% denr=1+e1*s+e2*s^2+e3*s^3; 						% desired denominator of the closed-loop
% numf=1+d1*s+d2*s^2+d3*s^3;						% Filter numerator : has to be CALCULATED! 

numcM= [];
KcM = [];
AreasM = [];
PIM= [];




% *****************************************************
%
%  Process 1  - Panda et al.
%
% *****************************************************


% Process
Kpr=2; Tdel=0.3; num=[0 1]; den=[-3 1]; den=conv(den,[-1 1]); %den=conv(den,[0 1]); 

% Compensator
order=2;Tf=0.1;denc=[Tf 1]; denc=conv(denc,denc); denc=conv(denc,[Tf 1]); %denc=conv(denc,[Tf 1]);

% Desired closed-loop denominator 
denr=[2 1]; denr=conv(denr,[1 1]); denr=conv(denr,[0 1]);


[numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);

% Only inner compensator

% Simulation parameters
Tscl = 0.01;
Tstopcl = 20;
Tdist = 0;
Kdist = 1;

% Controller parameters
K = 0; Ki = 0; b=1;

sim ('cl_unstab');

tol1 = tcl;
yol1 = ycl;
uol1 = ucl;
yref1 = yref;

% With PI controller

% Simulation parameters
Tscl = 0.01;
Tstopcl = 40;
Tdist = Tstopcl/2;
Kdist = 1;

% Controller parameters
[A0,A1,A2,A3,A4,A5] = numden2areas (Kpr/(1-Kpr*Kc)*num,denr,Tdel);
b=1;
[Ki,K] = area2PI (A0,A1,A2,A3,2);
Ki0=Ki;
K0 = K;
b0 = b;

sim ('cl_unstab');

tclr = tcl;
yclr = ycl;
uclr = ucl;

b=0;
[Ki,K] = area2PI_dist (A0,A1,A2,A3,2);

sim ('cl_unstab');

Ki1=Ki;
K1 = K;
b1 = b;

tcld = tcl;
ycld = ycl;
ucld = ucl;


figure(1);

subplot(211);
plot(tol1,yol1,tol1,yref1,'--');
grid
title('Process G_P_1 output: inner closed-loop response')
xlabel('time [s]')
legend('actual response','desired response',4)

subplot(212);
plot(tclr,yclr,tcld,ycld,'--');
grid
title('Process G_P_1 output with controller')
xlabel('time [s]')
legend('MOMI','DRMO')


print -depsc Gp_y_1.eps


while (max(size(numc)) < 3)
   numc = conv(numc,[0 1]);
end;
numcM = [numcM;numc];
KcM = [KcM;Kc];
AreasM = [AreasM;[A0,A1,A2,A3,A4,A5]];
PIM = [PIM;[Ki0,K0,b0,Ki1,K1,b1]];


% *****************************************************
%
%  Process 2  - Park et al., Jacob and Chidambaram
%
% *****************************************************


% Process
Kpr=4; Tdel=2; num=[1]; den=[-4 1]; %den=conv(den,[-1 1]); %den=conv(den,[0 1]); 

% Compensator
order=1;Tf=0.1; denc=[Tf 1]; denc=conv(denc,denc); %denc=conv(denc,[Tf 1]); %denc=conv(denc,[Tf 1]);

% Desired closed-loop denominator 
denr=[4 1]; %denr=conv(denr,[1 1]); denr=conv(denr,[1 1]);


[numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);

% Only inner compensator

% Simulation parameters
Tscl = 0.01;
Tstopcl = 30;
Tdist = 0;
Kdist = 1;

% Controller parameters
K = 0; Ki = 0; b=1;

sim ('cl_unstab');

tol1 = tcl;
yol1 = ycl;
uol1 = ucl;
yref1 = yref;

% With PI controller

% Simulation parameters
Tscl = 0.01;
Tstopcl = 80;
Tdist = Tstopcl/2;
Kdist = 1;

% Controller parameters
[A0,A1,A2,A3,A4,A5] = numden2areas (Kpr/(1-Kpr*Kc)*num,denr,Tdel);
b=1;
[Ki,K] = area2PI (A0,A1,A2,A3,2);
Ki0=Ki;
K0 = K;
b0 = b;

sim ('cl_unstab');

tclr = tcl;
yclr = ycl;
uclr = ucl;

b=0;
[Ki,K] = area2PI_dist (A0,A1,A2,A3,2);

sim ('cl_unstab');

Ki1=Ki;
K1 = K;
b1 = b;

tcld = tcl;
ycld = ycl;
ucld = ucl;


figure(2);

subplot(211);
plot(tol1,yol1,tol1,yref1,'--');
grid
title('Process G_P_2 output: inner closed-loop response')
xlabel('time [s]')
legend('actual response','desired response')

subplot(212);
plot(tclr,yclr,tcld,ycld,'--');
grid
title('Process G_P_2 output with controller')
xlabel('time [s]')
legend('MOMI','DRMO',4)


print -depsc Gp_y_2.eps


while (max(size(numc)) < 3)
   numc = conv(numc,[0 1]);
end;
numcM = [numcM;numc];
KcM = [KcM;Kc];
AreasM = [AreasM;[A0,A1,A2,A3,A4,A5]];
PIM = [PIM;[Ki0,K0,b0,Ki1,K1,b1]];


% *****************************************************
%
%  Process 3  - Prashanti et al.
%
% *****************************************************


% Process
Kpr=1; Tdel=1; num=[1]; den=[-2 1]; den=conv(den,[0.5 1]); %den=conv(den,[0 1]); 

% Compensator
order=2;Tf=0.1; denc=[Tf 1]; denc=conv(denc,denc); %denc=conv(denc,[Tf 1]); %denc=conv(denc,[Tf 1]);

% Desired closed-loop denominator 
denr=[1 1]; denr=conv(denr,[1 1]); %denr=conv(denr,[1 1]);


[numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);

% Only inner compensator

% Simulation parameters
Tscl = 0.01;
Tstopcl = 20;
Tdist = 0;
Kdist = 1;

% Controller parameters
K = 0; Ki = 0; b=1;

sim ('cl_unstab');

tol1 = tcl;
yol1 = ycl;
uol1 = ucl;
yref1 = yref;

% With PI controller

% Simulation parameters
Tscl = 0.01;
Tstopcl = 60;
Tdist = Tstopcl/2;
Kdist = 1;

% Controller parameters
[A0,A1,A2,A3,A4,A5] = numden2areas (Kpr/(1-Kpr*Kc)*num,denr,Tdel);
b=1;
[Ki,K] = area2PI (A0,A1,A2,A3,2);
Ki0=Ki;
K0 = K;
b0 = b;

sim ('cl_unstab');

tclr = tcl;
yclr = ycl;
uclr = ucl;

b=0;
[Ki,K] = area2PI_dist (A0,A1,A2,A3,2);

sim ('cl_unstab');

Ki1=Ki;
K1 = K;
b1 = b;

tcld = tcl;
ycld = ycl;
ucld = ucl;


figure(3);

subplot(211);
plot(tol1,yol1,tol1,yref1,'--');
grid
title('Process G_P_3 output: inner closed-loop response')
xlabel('time [s]')
legend('actual response','desired response')

subplot(212);
plot(tclr,yclr,tcld,ycld,'--');
grid
title('Process G_P_3 output with controller')
xlabel('time [s]')
legend('MOMI','DRMO',4)


print -depsc Gp_y_3.eps


while (max(size(numc)) < 3)
   numc = conv(numc,[0 1]);
end;
numcM = [numcM;numc];
KcM = [KcM;Kc];
AreasM = [AreasM;[A0,A1,A2,A3,A4,A5]];
PIM = [PIM;[Ki0,K0,b0,Ki1,K1,b1]];




% *****************************************************
%
%  Process 4  - Shamsuzzoha and Lee, 2008
%
% *****************************************************


% Process
Kpr=1; Tdel=0.4; num=[1]; den=[-1 1]; %den=conv(den,[-1 1]); %den=conv(den,[0 1]); 

% Compensator
order=1;Tf=0.05; denc=[Tf 1]; %denc=conv(denc,denc); %denc=conv(denc,[Tf 1]); %denc=conv(denc,[Tf 1]);

% Desired closed-loop denominator 
denr=[1 1]; %denr=conv(denr,[1 1]); denr=conv(denr,[1 1]);


[numc,Kc] = calc_comp_4 (Kpr,num,den,Tdel,denc,denr,order);

% Only inner compensator

% Simulation parameters
Tscl = 0.01;
Tstopcl = 12;
Tdist = 0;
Kdist = 1;

% Controller parameters
K = 0; Ki = 0; b=1;

sim ('cl_unstab');

tol1 = tcl;
yol1 = ycl;
uol1 = ucl;
yref1 = yref;

% With PI controller

% Simulation parameters
Tscl = 0.01;
Tstopcl = 20;
Tdist = Tstopcl/2;
Kdist = 1;

% Controller parameters
[A0,A1,A2,A3,A4,A5] = numden2areas (Kpr/(1-Kpr*Kc)*num,denr,Tdel);
b=1;
[Ki,K] = area2PI (A0,A1,A2,A3,2);
Ki0=Ki;
K0 = K;
b0 = b;

sim ('cl_unstab');

tclr = tcl;
yclr = ycl;
uclr = ucl;

b=0;
[Ki,K] = area2PI_dist (A0,A1,A2,A3,2);

sim ('cl_unstab');

Ki1=Ki;
K1 = K;
b1 = b;

tcld = tcl;
ycld = ycl;
ucld = ucl;


figure(4);

subplot(211);
plot(tol1,yol1,tol1,yref1,'--');
grid
title('Process G_P_4 output: inner closed-loop response')
xlabel('time [s]')
legend('actual response','desired response')

subplot(212);
plot(tclr,yclr,tcld,ycld,'--');
grid
title('Process G_P_4 output with controller')
xlabel('time [s]')
legend('MOMI','DRMO',4)


print -depsc Gp_y_4.eps


while (max(size(numc)) < 3)
   numc = conv(numc,[0 1]);
end;
numcM = [numcM;numc];
KcM = [KcM;Kc];
AreasM = [AreasM;[A0,A1,A2,A3,A4,A5]];
PIM = [PIM;[Ki0,K0,b0,Ki1,K1,b1]];


numcM
KcM
AreasM
PIM